/*******************************************************
Program: LSTSTRUC.PRG
Date   : 6 March 1996
Author : H. A. (Al) Fox
         CIS:71603,1742
         Internet:hafox@america.com
Purpose: To automatically print the structures of all
         DBF files in a directory, one per page, to
         a printer or file.
Notes  : Requires Frankie Library v3.x
Compile: clipper lststruc /n /m
Link   : blinker fi lststruc lib frankie

** This program is hereby placed in the public domain **
*******************************************************/


#include "dbstruct.ch"
#include "directry.ch"

#define STRUCFILE "STRUCLST.TXT"

static adbfiles  := {}
static adbstrucs := {}
static adbnames  := {}
static nPbel := 1
static nPbar := 0

func main()

local aCsr  := ADsavecsr(0)
local aScn1 := ADmessage("Scanning for DBF files in current directory",,,.f.,.t.)
local nSel  := 0
local nn    := 0

adbfiles := directory("*.dbf")
inkey(.5)  // Give the user a chance to actually read the messages.
ADrestscn( aScn1 )
if empty(adbfiles)
   ADmessage({"No DBF files could be found.",''})
   ADrestcsr(aCsr)
   return NIL
endif

nPbar := ADSbarCreate(len(adbfiles),;
                      {||ADSBarHeader("Obtaining structure info...")})
aeval(adbfiles,{|afile| GetStruc(afile[F_NAME])})
ADSbarkill(nPbar)
do while .t.
   nSel := ADboxmenu({"There are "+ADn2s(len(adbstrucs))+;
                      " structures to print.",;
                      "Ready to print?"},;
                     {" &Yes "," &No "," &File "})
   if nSel == 1 .or. nSel == 3
      if nSel == 3
         if file(STRUCFILE)
            if ADboxmenu("Overwrite existing "+STRUCFILE+"?",;
                         {" &Yes "," &No "})==2
               loop
            else
               erase (STRUCFILE)
            endif
         endif
         set printer to STRUCFILE additive
      else
         set printer to
      endif
      set printer on
      set console off
      aeval(adbstrucs,{|| PrintStruc(++nn)})
      set console on
      set printer off
      set printer to
   endif
   exit
enddo

ADrestcsr(aCsr)

return NIL

/*********************************************************/

//-----
procedure GetStruc(cfile)
local astruc := {}
local ldup := .f.
local nn := 0

ADSBarUpdate(nPBar,++nPbel)
use (cfile)
astruc := dbstruct()
close (cfile)
for nn := 1 to len(adbstrucs)
   if ADaequal(astruc,adbstrucs[nn])
      ldup := .t.
      exit
   endif
next
if !ldup
   aadd(adbstrucs,astruc)
   aadd(adbnames,{cfile})
else
   aadd(adbnames[nn],cfile)
endif
return

//-----
procedure PrintStruc(x)
local aScn := ADmessage("Printing structure of "+adbnames[x][1],,,.f.,.t.)
local xx
local nPage := 1
local nSize := 0

PHdgs(x,nPage)

for xx := 1 to len(adbstrucs[x])
   ?  str(xx,5,0)
   ?? '  '
   ?? padr(adbstrucs[x][xx][DBS_NAME],10)
   ?? '  '
   ?? padr(LongType(adbstrucs[x][xx][DBS_TYPE]),9)
   ?? '  '
   ?? padl(adbstrucs[x][xx][DBS_LEN],5)
   ?? '  '
   ?? padl(adbstrucs[x][xx][DBS_DEC],3)
   if (xx % 50) == 0
      ? '-More-'+chr(12)
      PHdgs(x,++nPage)
   endif
   nSize += adbstrucs[x][xx][DBS_LEN]
next // xx
? space(19)+'-Total-    '+padl(++nSize,5)
?? chr(12)
ADrestscn(aScn)
return

//-----
function LongType(cT)
local cLt := ''
do case
   case cT == 'C'
      cLt := 'Character'
   case cT == 'N'
      cLt := 'Numeric'
   case cT == 'M'
      cLt := 'Memo'
   case cT == 'D'
      cLt := 'Date'
   case cT == 'L'
      cLt := 'Logical'
   otherwise
      cLt := 'Unknown'
endcase
return cLt

//-----
procedure PHdgs(x,nPg)
local zz
?
? "Structure for file "
   for zz := 1 to len(adbnames[x])
      ?? padr(adbnames[x][zz],13)
      if (zz % 2) == 0 .and. zz < len(adbnames[x])
         ? space(19)
      endif
   next
? "Page "+ADn2s(nPg)
?
? "Field  Field Name  Type       Width  Dec"
? "----------------------------------------"
return
